@@ -36,20 +36,11 @@
 #' If the sufficient statistics are computed correctly then the
 #' results from \code{susie_suff_stat} should be the same as (or very
 #' similar to) \code{susie}, although runtimes will differ as
-#' discussed below. The simplest sufficient statistics are the sample
+#' discussed below. The sufficient statistics are the sample
 #' size \code{n}, and then the p by p matrix \eqn{X'X}, the p-vector
 #' \eqn{X'y}, and the sum of squared y values \eqn{y'y}, all computed
 #' after centering the columns of \eqn{X} and the vector \eqn{y} to
 #' have mean 0; these can be computed using \code{compute_suff_stat}.
-#' Alternatively the user can provide \code{n} and \code{bhat} (the
-#' univariate OLS estimates from regressing y on each column of X),
-#' \code{shat} (the standard errrors from these OLS regressions), the
-#' p by p symmetric, positive semidefinite correlation (or covariance)
-#' matrix \eqn{R = (1/(n-1))X'X}, and the variance of \eqn{y}, again
-#' all computed from centered \eqn{X} and \eqn{y}. Note that here
-#' \code{R} and \code{bhat} should be computed using the same matrix
-#' \eqn{X}. If you do not have access to the original \eqn{X} to
-#' compute the matrix \code{R} then use \code{\link{susie_rss}}.
 #'
 #' The handling of the intercept term in \code{susie_suff_stat} needs
 #' some additional explanation. Computing the summary data after
@@ -205,6 +196,9 @@
 #'  iterative refinement procedure is used, after the IBSS algorithm,
 #'  to check and escape from local optima (see details).
 #'
+#' @param n_purity Passed as argument \code{n_purity} to
+#'   \code{\link{susie_get_cs}}.
+#'
 #' @return A \code{"susie"} object with some or all of the following
 #'   elements:
 #'
@@ -262,6 +256,10 @@
 #'   to genetic fine-mapping. \emph{Journal of the Royal Statistical
 #'   Society, Series B} \bold{82}, 1273-1300 \doi{10.1101/501114}.
 #'
+#'   Y. Zou, P. Carbonetto, G. Wang and M. Stephens (2021).
+#'   Fine-mapping from summary data with the \dQuote{Sum of Single Effects}
+#'   model. \emph{bioRxiv} \doi{10.1101/2021.11.03.467167}.
+#'
 #' @seealso \code{\link{susie_get_cs}} and other \code{susie_get_*}
 #'   functions for extracting results; \code{\link{susie_trendfilter}} for
 #'   applying the SuSiE model to non-parametric regression, particularly
@@ -303,7 +301,7 @@ susie = function (X,y,L = min(10,ncol(X)),
                    scaled_prior_variance = 0.2,
                    residual_variance = NULL,
                    prior_weights = NULL,
-                   null_weight = NULL,
+                   null_weight = 0,
                    standardize = TRUE,
                    intercept = TRUE,
                    estimate_residual_variance = TRUE,
@@ -322,7 +320,8 @@ susie = function (X,y,L = min(10,ncol(X)),
                    verbose = FALSE,
                    track_fit = FALSE,
                    residual_variance_lowerbound = var(drop(y))/1e4,
-                   refine = FALSE) {
+                   refine = FALSE,
+                   n_purity = 100) {
 
   # Process input estimate_prior_method.
   estimate_prior_method = match.arg(estimate_prior_method)
@@ -346,9 +345,9 @@ susie = function (X,y,L = min(10,ncol(X)),
       prior_weights = c(prior_weights * (1-null_weight),null_weight)
     X = cbind(X,0)
   }
-  if (any(is.na(X)))
+  if (anyNA(X))
     stop("Input X must not contain missing values")
-  if (any(is.na(y))) {
+  if (anyNA(y)) {
     if (na.rm) {
       samples_kept = which(!is.na(y))
       y = y[samples_kept]
@@ -356,16 +355,31 @@ susie = function (X,y,L = min(10,ncol(X)),
     } else
       stop("Input y must not contain missing values")
   }
+  p = ncol(X)
+  if (p > 1000 & !requireNamespace("Rfast",quietly = TRUE))
+    warning_message("For an X with many columns, please consider installing",
+                    "the Rfast package for more efficient credible set (CS)",
+                    "calculations.", style='hint')
 
   # Check input y.
-  p = ncol(X)
   n = nrow(X)
   mean_y = mean(y)
 
   # Center and scale input.
   if (intercept)
     y = y - mean_y
-  X = set_X_attributes(X,center = intercept,scale = standardize)
+
+  # Set three attributes for matrix X: attr(X,'scaled:center') is a
+  # p-vector of column means of X if center=TRUE, a p vector of zeros
+  # otherwise; 'attr(X,'scaled:scale') is a p-vector of column
+  # standard deviations of X if scale=TRUE, a p vector of ones
+  # otherwise; 'attr(X,'d') is a p-vector of column sums of
+  # X.standardized^2,' where X.standardized is the matrix X centered
+  # by attr(X,'scaled:center') and scaled by attr(X,'scaled:scale').
+  out = compute_colstats(X,center = intercept,scale = standardize)
+  attr(X,"scaled:center") = out$cm
+  attr(X,"scaled:scale") = out$csd
+  attr(X,"d") = out$d
 
   # Initialize susie fit.
   s = init_setup(n,p,L,scaled_prior_variance,residual_variance,prior_weights,
@@ -376,15 +390,27 @@ susie = function (X,y,L = min(10,ncol(X)),
     if (max(s_init$alpha) > 1 || min(s_init$alpha) < 0)
       stop("s_init$alpha has invalid values outside range [0,1]; please ",
            "check your input")
+
     # First, remove effects with s_init$V = 0
-    s_init = susie_prune_single_effects(s_init, verbose=FALSE)
-    # Then prune or expand
-    s_init = susie_prune_single_effects(s_init, L, s$V, verbose)
+    s_init = susie_prune_single_effects(s_init)
+    num_effects = nrow(s_init$alpha)
+    if(missing(L)){
+      L = num_effects
+    }else if(min(p,L) < num_effects){
+      warning_message(paste("Specified number of effects L =",min(p,L),
+                    "is smaller than the number of effects",num_effects,
+                    "in input SuSiE model. The SuSiE model will have",
+                    num_effects,"effects."))
+      L = num_effects
+    }
+    # expand s_init if L > num_effects.
+    s_init = susie_prune_single_effects(s_init, min(p, L), s$V)
     s = modifyList(s,s_init)
     s = init_finalize(s,X = X)
   } else {
     s = init_finalize(s)
   }
+
   # Initialize elbo to NA.
   elbo = rep(as.numeric(NA),max_iter + 1)
   elbo[1] = -Inf;
@@ -445,22 +471,30 @@ susie = function (X,y,L = min(10,ncol(X)),
   # SuSiE CS and PIP.
   if (!is.null(coverage) && !is.null(min_abs_corr)) {
     s$sets = susie_get_cs(s,coverage = coverage,X = X,
-                          min_abs_corr = min_abs_corr)
+                          min_abs_corr = min_abs_corr,
+                          n_purity = n_purity)
     s$pip = susie_get_pip(s,prune_by_cs = FALSE,prior_tol = prior_tol)
   }
 
   if (!is.null(colnames(X))) {
     variable_names = colnames(X)
-    if (!is.null(null_weight))
-      variable_names = c("null", variable_names)
+    if (!is.null(null_weight)) {
+      variable_names[length(variable_names)] = "null"
+      names(s$pip) = variable_names[-p]
+    } else
+      names(s$pip)    = variable_names
     colnames(s$alpha) = variable_names
-    colnames(s$mu) = variable_names
-    colnames(s$mu2) = variable_names
+    colnames(s$mu)    = variable_names
+    colnames(s$mu2)   = variable_names
     colnames(s$lbf_variable) = variable_names
-    names(s$pip) = variable_names
   }
   # report z-scores from univariate regression.
   if (compute_univariate_zscore) {
+    if (!is.matrix(X))
+      warning("Calculation of univariate regression z-scores is not ",
+              "implemented specifically for sparse or trend filtering ",
+              "matrices, so this step may be slow if the matrix is large; ",
+              "to skip this step set compute_univariate_zscore = FALSE")
     if (!is.null(null_weight) && null_weight != 0)
       X = X[,1:(ncol(X) - 1)]
     s$z = calc_z(X,y,center = intercept,scale = standardize)
@@ -473,22 +507,25 @@ susie = function (X,y,L = min(10,ncol(X)),
     if (!missing(s_init) && !is.null(s_init))
       warning("The given s_init is not used in refinement")
     if (!is.null(null_weight) && null_weight != 0) {
-        
+
       ## if null_weight is specified, we compute the original prior_weight
       pw_s = s$pi[-s$null_index]/(1-null_weight)
       if (!compute_univariate_zscore)
-          
+
         ## if null_weight is specified, and the extra 0 column is not
         ## removed from compute_univariate_zscore, we remove it here
         X = X[,1:(ncol(X) - 1)]
     } else
       pw_s = s$pi
     conti = TRUE
-    while (conti) {
+    while (conti & length(s$sets$cs)>0) {
       m = list()
       for(cs in 1:length(s$sets$cs)){
         pw_cs = pw_s
         pw_cs[s$sets$cs[[cs]]] = 0
+        if(all(pw_cs == 0)){
+          break
+        }
         s2 = susie(X,y,L = L,scaled_prior_variance = scaled_prior_variance,
             residual_variance = residual_variance,
             prior_weights = pw_cs, s_init = NULL,null_weight = null_weight,
@@ -521,11 +558,15 @@ susie = function (X,y,L = min(10,ncol(X)),
             refine = FALSE)
         m = c(m,list(s3))
       }
-      elbo = sapply(m,function(x) susie_get_objective(x))
-      if ((max(elbo) - susie_get_objective(s)) <= 0)
+      if(length(m) == 0){
         conti = FALSE
-      else
-        s = m[[which.max(elbo)]]
+      }else{
+        elbo = sapply(m,function(x) susie_get_objective(x))
+        if ((max(elbo) - susie_get_objective(s)) <= 0)
+          conti = FALSE
+        else
+          s = m[[which.max(elbo)]]
+      }
     }
   }
   return(s)
